#ifndef _SYS_PRINT_H_
#define _SYS_PRINT_H_

/*
 * Copyright (C) 2001-2002 by egnite Software GmbH. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgement:
 *
 *    This product includes software developed by egnite Software GmbH
 *    and its contributors.
 *
 * THIS SOFTWARE IS PROVIDED BY EGNITE SOFTWARE GMBH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EGNITE
 * SOFTWARE GMBH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.ethernut.de/
 *
 */

/*
 * $Log: print.h,v $
 * Revision 1.6  2002/06/26 17:29:29  harald
 * First pre-release with 2.4 stack
 *
 */

#include <pgmspace.h>

#include <sys/device.h>

/*!
 * \file sys/print.h
 * \brief Formatted printing definitions.
 */

/**
 * \name Format flags
 *
 * \brief Number printing formats.
 *
 * These flags are used as number printing formats. Several
 * flags may be combined using the binary OR operator.
 *
 * \see NutPrintInteger
 *
 */
/*@{*/

#define FMTFLG_ZERO     0x01    /*!< \brief Set, if zero padding required */
#define FMTFLG_SIGNED   0x02    /*!< \brief Set, if signed value */
#define FMTFLG_PLUS     0x04    /*!< \brief Set to force sign */
#define FMTFLG_MINUS    0x08    /*!< \brief Set to force left justification */
#define FMTFLG_CAPITAL  0x10    /*!< \brief Set for capital letter digits */

/*@}*/

extern int NutPrintBinary(NUTDEVICE *dev, CONST char *data, int len);
extern int NutPrintBinary_P(NUTDEVICE *dev, PGM_P data, int len);
extern int NutPrintString(NUTDEVICE *dev, CONST char *str);
extern int NutPrintString_P(NUTDEVICE *dev, PGM_P str);
extern int NutPrintInteger(NUTDEVICE *dev, u_long val, u_char radix, u_char width, u_char flags);
extern int NutPrintFormat(NUTDEVICE *dev, CONST char *fmt, ...);
extern int NutPrintFlush(NUTDEVICE *dev);

extern HANDLE app1, app2, app3, app4;

#endif
